@@ -8,8 +8,6 @@ comm = MPI.COMM_WORLD
 
 
 
-
-
 def get_linear_indexes(mfp,value):
 
 
@@ -30,7 +28,7 @@ def get_linear_indexes(mfp,value):
 
 
 
-def generate_mfp3D(**argv): 
+def mfp3D(**argv): 
 
    #load data
    if argv.setdefault('read_from_file',True):
@@ -79,14 +77,6 @@ def generate_mfp3D(**argv):
    nm = n_phi * n_mfp * n_theta
    #------------------------------------------
 
-   #data = load_data('mfp')
-   #kappa_bulk = data['K']
-   #mfp_bulk = data['mfp']
-   #I = np.where(mfp_bulk > 0)
-   #kappa_bulk = kappa_bulk[I]
-   #mfp_bulk = mfp_bulk[I]
-   #kappa = data['kappa']
-   #-----------------------
 
    n_mfp_bulk = len(mfp_bulk)
 
