@@ -1,7 +1,8 @@
 from mpi4py import MPI
 from google_drive_downloader import GoogleDriveDownloader as gdd
 import numpy as np
-from shapely.geometry import Polygon
+from termcolor import colored, cprint 
+from shapely.geometry import Polygon,Point
 from shapely.geometry import MultiPolygon,LineString
 import shapely
 from shapely.ops import cascaded_union
@@ -13,6 +14,143 @@ import math
 import pickle
 import gzip
 
+
+
+def ggmres(L,X,compute_kappa,residual,k,rr,max_ggmres_iter,tolerance):
+
+      def compute_norm_D(A,D):
+       return np.sqrt(np.einsum('ij,i,ij->',A,D,A))
+
+      if comm.rank == 0:
+       print(' ')
+       print(colored('                        G-GMRES','green'),flush=True)
+       print(colored(' -----------------------------------------------------------','green'),flush=True)
+
+      h = np.zeros(k+1)
+
+      def arnoldi(Q,H):
+
+       for j in range(k):
+         #Arnoldi-------------------------------------
+         V         = L(Q[j]) #shared
+         hp         = np.einsum('ikj,k,kj->i',Q[:,rr,:],DD[rr],V[rr])
+         comm.Allreduce([np.array([hp]),MPI.DOUBLE],[h,MPI.DOUBLE],op=MPI.SUM)
+
+         H[:,j]    = h
+         V[rr]    -= np.einsum('ikj,i->kj',Q[:,rr],H[:,j])
+         comm.Barrier()
+         H[j+1,j]  = compute_norm_D(V,DD)
+         Q[j+1]    = V/H[j+1,j]   
+
+       return Q,H
+  
+      r = 1e6
+      kk = 0
+      while kk < max_ggmres_iter and r > tolerance:
+
+       X0 = X
+       R = residual(X0) #shared
+
+       #Compute WEIGHTS------------------------------
+       DD = np.mean(np.absolute(R),axis=1)
+       #---------------------------------------------  
+
+       R0   = compute_norm_D(R,DD)
+       Q = shared_array(np.zeros((k+1,X.shape[0],X.shape[1])))
+       Q[0] = R/R0
+       H = np.zeros((k+1,k))
+       beta2 = np.zeros(k+1);beta2[0] = R0
+
+       Q,H = arnoldi(Q,H)
+       #--------------
+       comm.Barrier() 
+       c   = np.linalg.lstsq(H,beta2)[0]
+       X   = shared_array(X0 + np.einsum('knm,k->nm',Q[:-1],c))
+       r   = np.linalg.norm(residual(X),ord='fro')
+       kappa = compute_kappa(X)
+     
+       if comm.rank == 0:
+        print('{0:8d} {1:24.4E} {2:22.4E}'.format(kk,kappa,r),flush=True)
+
+       kk +=1
+       Q[:,:,:] = 0
+
+      return X,kappa
+
+
+def cg(L, b,**argv):
+
+    #PARSE---------------------------------------
+    x  = argv.setdefault('x0',np.zeros_like(b))  
+    M  = argv.setdefault('M',lambda x: x.copy())  
+    max_error  = argv.setdefault('error',1e-4)  
+    callback  = argv.setdefault('callback',None) 
+    verbose  = argv.setdefault('verbose',True) 
+    
+    #-------------------------------------------
+    n = len(b)
+    r = b - L(x)
+    z = M(r)
+    p = z
+    r_k_norm = np.dot(r, z)
+    r_0_norm = r_k_norm
+    error = 1
+    for i in range(2*n):
+        Ap = L(p)
+        alpha = r_k_norm / np.dot(p, Ap)
+        x += alpha * p
+        r -= alpha * Ap
+        z = M(r)
+        r_kplus1_norm = np.dot(r, z)
+
+        error = r_kplus1_norm/r_0_norm
+        if not callback == None:
+            kappa = callback(x) 
+
+        if verbose:
+         print("Iter : % 4i, Kappa: % 5.3E  Error : % 5.2E" %(i, kappa,error))
+        if error < max_error:
+         break
+        if i == 100:
+            break
+
+        beta = r_kplus1_norm / r_k_norm
+        r_k_norm = r_kplus1_norm
+        p = z + beta * p
+    return x
+
+
+
+def fix_instability(F,B,scale=True):
+
+   n_elems = F.shape[0]
+   if scale:
+    scale = 1/F.max(axis=0).toarray()[0]
+   else: 
+    scale = np.ones(n_elems)   
+   n = np.random.randint(n_elems)
+   scale[n] = 0
+   F.data = F.data * scale[F.indices]
+   F[n,n] = 1
+   B[n] = 0   
+
+   return scale
+
+def compute_kappa(W,b):
+
+
+ L   = lambda x: np.dot(W,x)
+ M   = lambda x: x/np.sqrt(np.diag(W))
+
+ def callback(x):
+     return np.dot(x,b)
+
+ x0 = b.copy()/np.diag(W)
+ x  = cg(L,b,M=M,x0=x0,callback=callback,verbose = False)
+
+ return callback(x)
+
+
 os.environ['H5PY_DEFAULT_READONLY']='1'
 
 comm = MPI.COMM_WORLD
@@ -166,20 +304,18 @@ def create_line_list(pp,points,lines,store,lx,ly,step_label = 'h'):
 
 def save_data(fh, namedict):
 
-     with gzip.GzipFile(fh + '.npz', 'w') as f:
+     if comm.rank == 0:
+      with gzip.GzipFile(fh + '.npz', 'w') as f:
             pickle.dump(namedict, f,protocol=pickle.HIGHEST_PROTOCOL)
 
-     #pickle.sa
-     #with zipfile.ZipFile(fh + '.npz', mode="w", compression=zipfile.ZIP_DEFLATED,
-      #                    allowZip64=True) as zf:
-      #   for k, v in namedict.items():
-      #       with zf.open(k + '.npy', 'w', force_zip64=True) as buf:
-      #           np.lib.npyio.format.write_array(buf,
-                                                 #np.asanyarray(v),
-                                                 #allow_pickle=True)
+def load(fh):
+
+    return  {f:load_data(f) for f in fh}
+
 
 def load_data(fh):
 
+    if comm.rank == 0:
      if os.path.isfile(fh + '.npz'):
       with gzip.open(fh + '.npz', 'rb') as f:
           return pickle.load(f)
@@ -191,6 +327,46 @@ def download_file(file_id,filename):
       gdd.download_file_from_google_drive(file_id=file_id,
                                            dest_path='./' + filename,showsize=True,overwrite=True)
 
+
+
+def compute_neighbors(mesh,elem):
+  
+    guess = []
+
+    for side in mesh['elem_side_map_vec'][elem]:
+        for elem_2 in mesh['side_elem_map_vec'][side]:
+            if not elem_2 in guess:
+               guess.append(elem_2)
+               
+    return guess
+                
+
+
+def find_elem(mesh,p,guess):
+ 
+    #First try in guess--
+    for ne in guess:
+     elem  = mesh['elems'][ne][0:mesh['side_per_elem'][ne]] 
+     nodes = mesh['nodes'][elem][:,0:2] 
+     polygon = Polygon(mesh['nodes'][elem][0:mesh['side_per_elem'][ne],0:2])
+     if polygon.contains(Point(p[0],p[1])):
+      return ne,nodes[:,0],nodes[:,1]
+    #------------------      
+
+    #Find among all elements
+    for ne in range(len(mesh['elems'])):
+
+     elem  = mesh['elems'][ne][0:mesh['side_per_elem'][ne]]  
+     nodes = mesh['nodes'][elem][:,0:2]
+
+     polygon = Polygon(nodes)
+     if polygon.contains(Point(p[0],p[1])):
+      return ne,nodes[:,0],nodes[:,1]
+
+    return -1
+        
+
+
 def generate_frame(**argv):
 
 
@@ -469,6 +645,18 @@ def shared_array(value):
     return create_shared_memory_dict(data)['dummy']
 
 
+def get_kappa_map_from_mat(**argv):
+
+        mat_map = argv['geometry']['elem_mat_map']
+        kappa =   argv['material']['kappa']
+
+        kappa = np.array(kappa)
+        if kappa.ndim == 3:
+         return np.array([ list(kappa[i])  for i in mat_map])
+        else:
+         return np.array([ list(kappa)  for i in mat_map])
+            
+
 
 def create_shared_memory_dict(varss):
 
@@ -478,7 +666,6 @@ def create_shared_memory_dict(varss):
           for var,value in varss.items():
 
            if type(value) == list: value = np.array(value)   
-           
            if value.dtype == np.int64:
               data_type = 0
               itemsize = MPI.INT.Get_size()
